import nest
import numpy as np
import parameters as p
import nest.raster_plot
import nest.voltage_trace
import matplotlib.pyplot as plt

####### NEST #######

nest.ResetKernel()
nest.SetKernelStatus({'resolution': 0.01})
nest.SetDefaults('voltmeter', {'interval': 0.01})

neuron_pre = nest.Create('iaf_psc_alpha')
neuron_post = nest.Create('iaf_psc_alpha')
spikegenerator = nest.Create('spike_generator')
voltmeter_pre = nest.Create('voltmeter')
voltmeter_post = nest.Create('voltmeter')
spike_pre = nest.Create('spike_detector')
spike_post = nest.Create('spike_detector')
spike_gen = nest.Create('spike_detector')
wr = nest.Create('weight_recorder')
vt = nest.Create("volume_transmitter")

nest.CopyModel("stdp_dopamine_synapse", "rstdp",
                {"weight_recorder": wr[0], 
                "vt": vt[0],
                "tau_c": p.tau_c, 
                "tau_n": p.tau_n, 
                "Wmin": p.w_min, 
                "Wmax": p.w_max, 
                "A_plus": p.A_plus, 
                "A_minus": p.A_minus,
                "weight": 1e3,
                "b": 1.0})

nest.SetStatus(spikegenerator, {'spike_times': np.arange(5., 100., 5.)})

nest.Connect(spikegenerator, neuron_pre, syn_spec={'weight': 1e3})
nest.Connect(neuron_pre, neuron_post, syn_spec='rstdp')
nest.Connect(voltmeter_pre, neuron_pre)
nest.Connect(voltmeter_post, neuron_post)
nest.Connect(neuron_pre, spike_pre)
nest.Connect(neuron_post, spike_post)
nest.Connect(spikegenerator, spike_gen)

conn = nest.GetConnections(source=neuron_pre, target=neuron_post)

c_values = []

for i in range(1000):
    nest.Simulate(0.1)
    c_values.append(nest.GetStatus(conn, 'c')[0])

####### Brian2 #######

from brian2 import *

defaultclock.dt = 0.01*ms

#=========== ======  ==========================================================
# V_m        mV      Membrane potential
# C_m        pF      Capacity of the membrane
# E_L        mV      Resting membrane potential
# tau_m      ms      Membrane time constant
# t_ref      ms      Duration of refractory period
# V_th       mV      Spike threshold
# V_reset    mV      Reset potential of the membrane
# tau_syn_ex ms      Rise time of the excitatory synaptic alpha function
# tau_syn_in ms      Rise time of the inhibitory synaptic alpha function
# I_e        pA      Constant input current
#=========== ======  ==========================================================
neuron_equation = """
dV_m/dt = (E_L-V_m)/tau_m + (exp(1) * I + I_e)/C_m : volt (unless refractory)
dI/dt = (x-I)/tau_syn_ex : amp
dx/dt = -x/tau_syn_ex    : amp
I_e : amp
"""

C_m = 250.0 * pF
tau_m = 10.0 * ms
tau_syn_ex = 2.0 * ms
tau_syn_in = 2.0 * ms 
E_L = -70.0 * mV
V_reset = -70.0 * mV
V_th = -55.0 * mV

times = arange(5., 100., 5.) *ms
indices = zeros(times.shape)
parrot_neuron = SpikeGeneratorGroup(1, indices, times)

input_neuron = NeuronGroup(1, neuron_equation, threshold='V_m > V_th', reset='V_m = V_reset', refractory=2*ms, method='exact')
input_neuron.V_m = E_L
input_neuron.I_e = 0*pA

output_neuron = NeuronGroup(1, neuron_equation, threshold='V_m > V_th', reset='V_m = V_reset', refractory=2*ms, method='exact')
output_neuron.V_m = E_L
output_neuron.I_e = 0*pA

input_conn = Synapses(parrot_neuron, input_neuron, 'w : amp', on_pre='x_post += w', delay=1*ms)
input_conn.connect()
input_conn.w = 1e3 * pA

rstdp_eq = """
dn/dt = -n/tau_n : 1 (clock-driven)
dc/dt = -c/tau_c : 1 (clock-driven)
dweight/dt = c*(n-b) / (1*ms) : 1 (clock-driven)
dApre/dt = -Apre/tau_plus : 1 (clock-driven)
dApost/dt = -Apost/tau_minus : 1 (clock-driven)
b : 1
A_plus : 1
A_minus : 1
tau_n : second
tau_c : second
tau_plus : second
tau_minus : second
"""

rstdp_pre_eq = """
x_post += weight*pA
c += Apost
Apre += A_plus
"""

rstdp_post_eq = """
c += Apre
Apost += A_minus
"""

output_conn = Synapses(input_neuron, output_neuron, rstdp_eq, on_pre=rstdp_pre_eq, on_post=rstdp_post_eq, delay=1*ms)
output_conn.connect()
output_conn.b = 1.0
output_conn.c = 0.0
output_conn.A_plus = p.A_plus
output_conn.A_minus = -p.A_minus
output_conn.tau_n = p.tau_n * ms
output_conn.tau_c = p.tau_c * ms
output_conn.tau_plus = 20.0 * ms
output_conn.tau_minus = 20.0 * ms
output_conn.weight = 1e3

input_voltage_monitor = StateMonitor(input_neuron, ['V_m', 'I'], record=True)
output_voltage_monitor = StateMonitor(output_neuron, 'V_m', record=True)
weight_monitor = StateMonitor(output_conn, ['weight', 'n', 'c'], record=True)

run(100*ms)
#output_conn.n = 1.0
#run(50*ms)

####### Plot results #######

fig = figure()
ax1 = fig.add_subplot(411)
nest.voltage_trace.from_device(voltmeter_pre)
plot(input_voltage_monitor.t/ms, input_voltage_monitor.V_m[0]/mV, '--')

legend(['Nest input', 'Brian2 input'])
gca().set_xlim(0, 50)
grid('on')
ax1.title.set_text('Input neuron membrane potential')

ax2 = fig.add_subplot(412)
nest.voltage_trace.from_device(voltmeter_post)
plot(output_voltage_monitor.t/ms, output_voltage_monitor.V_m[0]/mV, '--')

legend(['Nest output', 'Brian2 output'])
gca().set_xlim(0, 50)
grid('on')
ax2.title.set_text('Output neuron membrane potential')

ax3 = fig.add_subplot(413)
weights = nest.GetStatus(wr, "events")[0]["weights"]
times = nest.GetStatus(wr, "events")[0]["times"]
plot(times, weights)
plot(weight_monitor.t/ms, weight_monitor.weight[0], '--')

legend(['Nest', 'Brian2'])
gca().set_xlim(0, 50)
grid('on')
ax3.title.set_text('Weight')

ax4 = fig.add_subplot(414)
plt.plot(arange(0, 100, 0.1), c_values)
plot(weight_monitor.t/ms, weight_monitor.c[0], '--')

legend(['Nest', 'Brian2'])
gca().set_xlim(0, 50)
grid('on')
ax4.title.set_text('Eligibility trace')

show()

"""
plt.subplot(3,1,1)

evt = spike_pre.get("events")
plt.plot(evt["times"], evt["senders"], '.')

evt = spike_post.get("events")
plt.plot(evt["times"], evt["senders"], '.')

evt = spike_gen.get("events")
plt.plot(evt["times"], evt["senders"], '.')

plt.gca().set_xlim(0, 100)
plt.gca().set_ylim(0, 4)
plt.grid('on')

plt.subplot(3,1,2)
nest.voltage_trace.from_device(voltmeter_pre)
nest.voltage_trace.from_device(voltmeter_post)

plt.plot(input_voltage_monitor.t/ms, input_voltage_monitor.V_m[0]/mV, '--')
plt.plot(output_voltage_monitor.t/ms, output_voltage_monitor.V_m[0]/mV, '--')

plt.gca().set_xlim(0, 100)
plt.grid('on')

plt.subplot(3,1,3)
weights = nest.GetStatus(wr, "events")[0]["weights"]
times = nest.GetStatus(wr, "events")[0]["times"]
plt.plot(times, weights)

plt.plot(c_values)
plt.plot(weight_monitor.t/ms, weight_monitor.c[0], '--')

plt.gca().set_xlim(0, 100)
plt.grid('on')


figure()
#plt.plot(arange(0,100,0.5), c_values)
#plt.plot(weight_monitor.t/ms, weight_monitor.n[0], '--')
#plt.plot(weight_monitor.t/ms, weight_monitor.c[0], '--')
plt.plot(input_voltage_monitor.t/ms, input_voltage_monitor.I[0]/pA, '--')

plt.show()
"""
## save data
"""
np.savetxt("input_V.csv", nest.GetStatus(voltmeter_pre, 'events')[0]['V_m'], delimiter=",")
np.savetxt("output_V.csv", nest.GetStatus(voltmeter_post, 'events')[0]['V_m'], delimiter=",")

evt = spike_pre.get("events")
np.savetxt("pre_spike.csv", evt["times"])

evt = spike_post.get("events")
np.savetxt("post_spike.csv", evt["times"])

evt = spike_gen.get("events")
np.savetxt("gen_spike.csv", evt["times"])

weights = nest.GetStatus(wr, "events")[0]["weights"]
np.savetxt("weights.csv", weights)

np.savetxt("trace.csv", c_values)
"""